import { geoAlbersUsa, geoAlbers, geoPath, geoGraticule } from 'd3';
import { useMemo } from 'react';

//const projection = geoAlbers();
const projection = geoAlbersUsa();
const pathProj = geoPath(projection);
const graticule = geoGraticule();

const missingDataColor = 'gray';

export const MapMarks = ({
  data,
	dataMap,
  atlas: { states, interiors },
  colorScale,
  colorValue,
  tooltipValue,
  tooltipFormat,
  onClick,
  selectedValue,
  onHover,
  hoveredValue,
  hoverOpacity,
  fadeOpacity
}) => (
  <g className="marks">
    {useMemo(
      () => (
        <>
          <path className="sphere" d={pathProj({ type: 'Sphere' })} />
          <path className="graticules" d={pathProj(graticule())} />
          <path className="interiors" d={pathProj(interiors)} />
        </>
      ),
      [pathProj, graticule, interiors]
    )}
    {useMemo(
      () =>
        states.features.map(feature => {
          const yId = feature.properties.name;
          //console.log('featureName:', yId);
          const stateColor = obj => {
            return data
              .filter(d => d.State === obj.properties.name)
              .map(d => colorScale(colorValue(d)));
          };
          //stateColor(feature) no longer used!
          const d = dataMap.get(feature.properties.name);
          if(!d) {
            console.log(feature.properties.name);
          }
          return (
            <path
              className="land-state"
              d={pathProj(feature)}
              fill={d ? colorScale(d) : missingDataColor}
              onClick={() => {
          console.log('clicked:', yId);
                onClick(yId);
              }}
              onMouseEnter={() => {
          console.log('hovered:', yId);
                onHover(yId);
              }}
              onMouseOut={() => {
                onHover(null);
              }}
              opacity={
                yId === selectedValue || yId === hoveredValue
                  ? hoverOpacity
                  : fadeOpacity
              }
              >
          <title>{yId+':'+tooltipFormat(dataMap.get(yId))}</title>
            </path>
          );
        }),
      [pathProj, states, data]
    )}
  </g>
);
