import {
  getLabel,
  ValueAttributes,
  FieldAttributes,
  NumOfStates
} from './AttributesDefs';
import { format, zoom } from 'd3';
import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import ReactDropdown from 'react-dropdown';
import { config } from './config';
import { useData } from './useData';
import { getTop10BarData } from './sumData';
import { getSumData } from './sumDataByYear';
import { getSumData2, getSumDataAll } from './sumDataByState';
import { getUsAtlas } from './getUsAtlas';
import { BarColorMark } from './BarColorMark/index.js';
import { DotColorMark } from './DotColorMark/index.js';
import { MapColorMark } from './MapColorMark/index.js';
import { LineColorMark } from './LineColorMark/index.js';

//Part I: Global values
const width = 960;
const menuHeight = 60;
const height = 500 - menuHeight;
const margin = { top: 10, right: 30, bottom: 65, left: 90 };
const xAxisLabelOffset = 30;
const yAxisLabelOffset = 50;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
const halfWidth = (innerWidth - 20) / 2;
const halfHeight = (innerHeight - 20) / 2;
const halfMarginX = 30;
const halfMarginY = 40;
const leftMarginX = 140;
//Interaction related
const fadeOpacity = 0.2;
const normalOpacity = 0.8;
const highlightOpacity = 0.96;

//Part II: Main functiond
const App = () => {
  const data = useData();
  const usAtlas = getUsAtlas();
  const [hoveredYearValue, setHoveredYearValue] = useState(null);
  const [hoveredStateValue, setHoveredStateValue] = useState(null);
  const [selectYearValue, setSelectedYearValue] = useState(null);
  const [selectStateValue, setSelectedStateValue] = useState(null);

  const handleResetButtonClick = e => {
    e.preventDefault();
    setSelectedYearValue(null);
    setSelectedStateValue(null);
    console.log('The link was clicked.');
  };

  //Step 0: Menu with attribute selections
  //xValue and yValues are defined here
  const initialXAttribute = 'Week';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  //  xLabel is fixed for X
  const xAxisLabel = getLabel(xAttribute, FieldAttributes);

  const initialYAttribute = 'ILI_Percent';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute, ValueAttributes);

  const initialNumState = '10';
  const [topStateNum, setTopStateNum] = useState(initialNumState);

  if (!data || !usAtlas) {
    return <pre>Loading...</pre>;
  }
  //console.log(yAxisLabel);
  //console.log(xAxisLabel);
  //console.log(xAttribute);

  const siFormat = format('.2g');
  //const siFormat1 = format('.2s');
  const siFormat2 = format('4d');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');
  const xAxisTickFormatYear = tickValue => siFormat2(tickValue).replace('G', 'B');
  //hoverOpacity value definitions
  const hoverOpacityYear =
    hoveredYearValue !== null ? highlightOpacity : normalOpacity;
  const fadeOrNormalOpacityYear =
    hoveredYearValue !== null ? fadeOpacity : normalOpacity;
  const hoverOpacityState =
    hoveredStateValue !== null ? highlightOpacity : normalOpacity;
  const fadeOrNormalOpacityState =
    hoveredStateValue !== null ? fadeOpacity : normalOpacity;

  // console.log(
  //   'hover: ',
  //   hoveredYearValue,
  //   hoveredStateValue,
  //   fadeOrNormalOpacity,
  //   hoverOpacity
  // );
  // console.log('selected: ', selectYearValue, selectStateValue);

  //Step 1: Derived data from here for plot:4-1 Line chart
  //Move into LineColorMark
  const colorValue = d => d.Year;
  const sizeValue = d => d.ILI_Total;
  const tooltipValue = d => d.ILI_Total;
  const tooltipState = d => d;
  const tooltipYear = d => d;
  const color2ndValue = d => d.State;
  const sumDataByYear = getSumData(data, yValue);
  console.log('Called sumData by year.');

  //Step 2: US map for all states plot: 4-2
  //Move into MapColorMark\
  //console.log(usAtlas);
  //obviously svg is not available here!
  //  const svg = select('svg');

  //Step 3: Derived data from here for plot: 4-3 Bar chart
  const top10BarData = getTop10BarData(data, yValue, topStateNum);
  //  console.log(top10BarData);
  //Move into BarColorMark\
  console.log('Called getTop10Bar by year.');

  const yBarValue = d => d.State;
  const xBarValue = d => d.Tvalue;

  //Step 4: Derived line plot plot: 4-4 Curve chart
  //  const xAxisTickFormat = timeFormat('%a');
  //Aggregate data by weeks
  // const topStates = top10BarData.map(d => (d.State));
  // console.log(topStates);
  const top10States = {};
  top10BarData.forEach(d => (top10States[d.State] = true));
  //  console.log(topStates);
  const top10Data = data.filter(d => top10States[d['State']]);
  const sumData10State = getSumData2(top10Data, yValue, color2ndValue);
  console.log('Called getSumData2 by state.');

  const sumDataAllState = getSumDataAll(data, yValue, color2ndValue);
  console.log(yValue, yAttribute);
  console.log(color2ndValue);
  const stateToValue = d => d.avg;

  //Step 5: Draw plots using svg, other controls: ReactDropdown
  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">Choose a data field:</span>
        <ReactDropdown
          options={ValueAttributes}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
        <span className="dropdown-label"> </span>
        <button className="button" onClick={handleResetButtonClick}>
          {' '}
          Reset{' '}
        </button>
        <span className="dropdown-label">Top N States:</span>
        <ReactDropdown
          options={NumOfStates}
          value={topStateNum}
          onChange={({ value }) => setTopStateNum(value)}
        />
      </div>
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left + halfWidth + halfMarginX},${
            margin.top
          }) scale(0.995 0.995)`}
        >
          <svg
            width={halfWidth}
            height={halfHeight + 20}
            viewBox={`0 0 ${width} ${height + 60}`}
            zoomAndPan={'zoomAndPan'}
            zoom={zoom().on('zoom', () => {
              g.attr('transform', event.transform);
            })}
          >
            <MapColorMark
              data={sumDataAllState}
              atlas={usAtlas}
              avgValue={stateToValue}
              colorValue={color2ndValue}
              innerWidth={halfWidth}
              innerHeight={halfHeight}
              tooltipValue={yBarValue}
              tooltipFormat={xAxisTickFormat}
              setSelectedValue={setSelectedStateValue}
              selectedValue={selectStateValue}
              setHoveredValue={setHoveredStateValue}
              hoveredValue={hoveredStateValue}
              hoverOpacity={'0.45'}
              fadeOpacity={'1.0'}
            />
          </svg>
        </g>
        <g
          transform={`translate(${margin.left + 5},${margin.top +
            halfHeight +
            halfMarginY})`}
        >
          <BarColorMark
            data={top10BarData}
            xValue={xBarValue}
            yValue={yBarValue}
            colorValue={color2ndValue}
            innerWidth={halfWidth}
            innerHeight={halfHeight}
            xAxisLabel={yAxisLabel}
            xAxisTickFormat={xAxisTickFormat}
            tooltipFormat={tooltipState}
            setSelectedValue={setSelectedStateValue}
            selectedValue={selectStateValue}
            setHoveredValue={setHoveredStateValue}
            hoveredValue={hoveredStateValue}
            hoverOpacity={normalOpacity}
            fadeOpacity={fadeOrNormalOpacityState}
          />
        </g>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <LineColorMark
            data={sumDataByYear}
            xValue={xValue}
            yValue={yValue}
            sizeValue={sizeValue}
            colorValue={colorValue}
            innerWidth={halfWidth}
            innerHeight={halfHeight}
            xAxisLabel={xAxisLabel}
            yAxisLabel={yAxisLabel}
            xAxisTickFormat={xAxisTickFormatYear}
            tooltipValue={tooltipValue}
            setSelectedValue={setSelectedYearValue}
            selectedValue={selectYearValue}
            setHoveredValue={setHoveredYearValue}
            hoveredValue={hoveredYearValue}
            hoverOpacity={hoverOpacityYear}
            fadeOpacity={fadeOrNormalOpacityYear}
          />
        </g>
        <g
          transform={`translate(${margin.left +
            2 +
            halfWidth +
            halfMarginX +
            leftMarginX},${margin.top + halfHeight + halfMarginY})`}
        >
          <LineColorMark
            data={sumData10State}
            xValue={colorValue}
            yValue={yValue}
            sizeValue={sizeValue}
            colorValue={color2ndValue}
            colorDomain={top10BarData}
            innerWidth={halfWidth - leftMarginX}
            innerHeight={halfHeight}
            legendOnLeft={'true'}
            xAxisLabel={'Year'}
            yAxisLabel={yAxisLabel}
            xAxisTickFormat={xAxisTickFormatYear}
            tooltipValue={tooltipValue}
            setSelectedValue={setSelectedStateValue}
            selectedValue={selectStateValue}
            setHoveredValue={setHoveredStateValue}
            hoveredValue={hoveredStateValue}
            hoverOpacity={hoverOpacityState}
            fadeOpacity={fadeOrNormalOpacityState}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
